@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
 """
 See PEP 386 (https://www.python.org/dev/peps/pep-0386/)
 
@@ -12,13 +10,15 @@ Release logic:
  6. git push
  7. assure that all tests pass on https://travis-ci.org/jrief/djangocms-cascade
  8. git push --tags
- 9. python setup.py sdist upload
-10. bump the version, append ".dev0" to __version__
-11. Add a new heading to docs/source/changelog.rst, named "<next-version>.dev0"
-12. git add cmsplugin_cascade/__init__.py docs/source/changelog.rst
-12. git commit -m 'Start with <version>'
-13. git push
+ 9. rm -rf dist
+10. python setup.py sdist
+11. twine upload dist/*
+12. bump the version, append ".dev0" to __version__
+13. Add a new heading to docs/source/changelog.rst, named "<next-version>.dev0"
+14. git add cmsplugin_cascade/__init__.py docs/source/changelog.rst
+15. git commit -m 'Start with <version>'
+16. git push
 """
-__version__ = "0.11.3"
+__version__ = "1.2.1"
 
 default_app_config = 'cmsplugin_cascade.apps.CascadeConfig'
