@@ -1,12 +1,10 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
+import sys
 from importlib import import_module
 from django.core.exceptions import ImproperlyConfigured
-from .settings import CASCADE_PLUGINS
+from . import app_settings
 
 
-for module in CASCADE_PLUGINS:
+for module in app_settings.CASCADE_PLUGINS:
     try:
         # if a module was specified, load all plugins in module settings
         module_settings = import_module('{}.settings'.format(module))
@@ -15,8 +13,9 @@ for module in CASCADE_PLUGINS:
             try:
                 import_module('{}.{}'.format(module, p))
             except ImportError as err:
-                msg = "Plugin {} as specified in {}.settings.CMSPLUGIN_CASCADE_PLUGINS could not be loaded: {}"
-                raise ImproperlyConfigured(msg.format(p, module, err))
+                traceback = sys.exc_info()[2]
+                msg = "Plugin {} as specified in {}.settings.CASCADE_PLUGINS could not be loaded: {}"
+                raise ImproperlyConfigured(msg.format(p, module, err.with_traceback(traceback)))
     except ImportError:
         try:
             # otherwise try with cms_plugins in the named module
