@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 
 class PluginExtraFieldsConfig(object):
     """
@@ -8,19 +5,22 @@ class PluginExtraFieldsConfig(object):
     CSS classes or inlines styles. It is possible to configure these fields globally using an
     instance of this class, or to configure them on a per site base using the appropriate
     admin's backend interface at:
-
+    ```
     *Start › django CMS Cascade › Custom CSS classes and styles › PluginExtraFields*
+    ```
+    :param allow_id_tag:
+        If ``True``, allows to set the ``id`` attribute in HTML elements.
 
-    :param allow_id_tag: If ``True``, allows to set the ``id`` attribute in HTML elements.
+    :param css_classes:
+        A dictionary containing:
+          ``class_names`` a comma separated string of allowed class names.
+          ``multiple`` a Boolean indicating if more multiple classes are allowed concurrently.
 
-    :param css_classes: A dictionary containing:
-        ``class_names`` a comma separated string of allowed class names.
-        ``multiple`` a Boolean indicating if more multiple classes are allowed concurrently.
+    :param inline_styles:
+        A dictionary containing:
 
-    :param inline_styles: A dictionary containing:
-
-    :param allow_override: If ``True``, allows to override this configuration using the admin's
-    backend interface.
+    :param allow_override:
+        If ``True``, allows to override this configuration using the admin's backend interface.
     """
     def __init__(self, allow_id_tag=False, css_classes=None, inline_styles=None, allow_override=True):
         self.allow_id_tag = allow_id_tag
