@@ -1,16 +1,13 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 from django.utils.html import mark_safe, format_html_join
+from django.utils.functional import cached_property
+import json
 from jsonfield.fields import JSONField
 from cms.models import CMSPlugin
 from cms.plugin_pool import plugin_pool
 from cms.utils.placeholder import get_placeholder_conf
 
 
-@python_2_unicode_compatible
 class CascadeModelBase(CMSPlugin):
     """
     The container to hold additional HTML element tags.
@@ -18,17 +15,21 @@ class CascadeModelBase(CMSPlugin):
     class Meta:
         abstract = True
 
-    cmsplugin_ptr = models.OneToOneField(CMSPlugin, related_name='+', parent_link=True)
+    cmsplugin_ptr = models.OneToOneField(
+        CMSPlugin,
+        related_name='+',
+        on_delete=models.CASCADE,
+        parent_link=True,
+    )
+
     glossary = JSONField(blank=True, default={})
 
     def __str__(self):
         return self.plugin_class.get_identifier(self)
 
-    @property
+    @cached_property
     def plugin_class(self):
-        if not hasattr(self, '_plugin_class'):
-            self._plugin_class = self.get_plugin_class()
-        return self._plugin_class
+        return self.get_plugin_class()
 
     @property
     def tag_type(self):
@@ -47,24 +48,36 @@ class CascadeModelBase(CMSPlugin):
     @property
     def html_tag_attributes(self):
         attributes = self.plugin_class.get_html_tag_attributes(self)
-        return format_html_join(' ', '{0}="{1}"', ((attr, val) for attr, val in attributes.items() if val))
+        joined = format_html_join(' ', '{0}="{1}"', ((attr, val) for attr, val in attributes.items() if val))
+        if joined:
+            return mark_safe(' ' + joined)
+        return ''
 
-    def get_parent(self):
-        raise NotImplementedError("This method is deprecated. Use `get_parent_instance` instead.")
+    def get_parent_instance(self):
+        for model in CascadeModelBase._get_cascade_elements():
+            try:
+                return model.objects.get(id=self.parent_id)
+            except model.DoesNotExist:
+                continue
+        # in case our plugin is the child of a TextPlugin, return its grandparent
+        parent = self.get_parent()
+        if parent and parent.plugin_type == 'TextPlugin':
+            grandparent_id = self.get_parent().parent_id
+            for model in CascadeModelBase._get_cascade_elements():
+                try:
+                    return model.objects.get(id=grandparent_id)
+                except model.DoesNotExist:
+                    continue
 
     def get_parent_glossary(self):
         """
         Return the glossary from the parent of this object. If there is no parent, retrieve
         the glossary from the placeholder settings, if configured.
         """
-        for model in CascadeModelBase._get_cascade_elements():
-            try:
-                parent = model.objects.get(id=self.parent_id)
-            except model.DoesNotExist:
-                continue
-            else:
-                return parent.get_complete_glossary()
-        # use self.placeholder.glossary as the starting dictionary
+        parent = self.get_parent_instance()
+        if parent:
+            return parent.get_complete_glossary()
+        # otherwise use self.placeholder.glossary as the starting dictionary
         template = self.placeholder.page.template if self.placeholder.page else None
         return get_placeholder_conf('glossary', self.placeholder.slot, template=template, default={})
 
@@ -79,6 +92,12 @@ class CascadeModelBase(CMSPlugin):
             self._complete_glossary_cache.update(self.glossary or {})
         return self._complete_glossary_cache
 
+    def get_num_children(self):
+        """
+        Returns the number of children for this plugin instance.
+        """
+        return self.get_children().count()
+
     def sanitize_children(self):
         """
         Recursively walk down the plugin tree and invoke method ``save(sanitize_only=True)`` for
@@ -90,7 +109,14 @@ class CascadeModelBase(CMSPlugin):
             for child in children:
                 child.save(sanitize_only=True)
                 child.sanitize_children()
-
+                
+    @classmethod
+    def from_db(cls, db, field_names, values):
+        instance = cls(*values)
+        if isinstance(instance.glossary, str):
+           instance.glossary = json.loads(instance.glossary)
+        return instance
+                
     def save(self, sanitize_only=False, *args, **kwargs):
         """
         A hook which let the plugin instance sanitize the current object model while saving it.
@@ -100,15 +126,9 @@ class CascadeModelBase(CMSPlugin):
         sanitized = self.plugin_class.sanitize_model(self)
         if sanitize_only:
             if sanitized:
-                super(CascadeModelBase, self).save(no_signals=True)
+                super().save(no_signals=True)
         else:
-            super(CascadeModelBase, self).save(*args, **kwargs)
-
-    def get_data_representation(self):
-        """
-        Hook to return a serializable representation of this element.
-        """
-        return {'glossary': self.glossary}
+            super().save(*args, **kwargs)
 
     @classmethod
     def _get_cascade_elements(cls):
