@@ -2,18 +2,16 @@
 django.jQuery(function($) {
 	'use strict';
 
-	var ImagePlugin,
-	    $image_responsive = $('#id_glossary_image_shapes_0'),
-	    base_plugins = eval(django.cascade.base_plugins);
+	var $image_responsive = $('#id_image_shapes_0');
 
 	// create class handling the client-side part of ImagePlugin
-	ImagePlugin = ring.create(base_plugins, {
+	django.cascade.ImagePlugin = ring.create(eval(django.cascade.ring_plugin_bases.ImagePlugin), {
 		constructor: function() {
 			var self = this;
 			this.$super();
 
 			// be more intuitive, reorganize layout by moving 'Link Target'
-			$('.glossary-widget .glossary_target').before($('.form-row.field-link_type'));
+			// $('.glossary-widget .glossary_target').before($('.form-row.field-link_type'));
 
 			// install event handlers
 			$image_responsive.change(function(evt) {
@@ -22,16 +20,19 @@ django.jQuery(function($) {
 			this.refreshChangeForm();
 		},
 		toggleResponsive: function(checked) {
-			var $image_width_responsive = $('#id_glossary_image_width_responsive').closest('.glossary-widget'),
-				$image_width_fixed = $('#id_glossary_image_width_fixed').closest('.glossary-widget');
+			var $image_width_responsive = $('.form-row.field-image_width_responsive'),
+				$image_width_fixed = $('.form-row.field-image_width_fixed'),
+				$image_alignment = $('.form-row.field-image_alignment');
 
 			// if checkbox Image Shapes: Responsive is active, show adaptive width and heights
 			if (checked) {
 				$image_width_responsive.show();
 				$image_width_fixed.hide();
+				$image_alignment.hide();
 			} else {
 				$image_width_responsive.hide();
 				$image_width_fixed.show();
+				$image_alignment.show();
 			}
 		},
 		refreshChangeForm: function() {
@@ -39,6 +40,4 @@ django.jQuery(function($) {
 			this.$super && this.$super();
 		}
 	});
-
-	new ImagePlugin();
 });
