@@ -12,7 +12,7 @@ django.jQuery(function($) {
 	$('#result_list a.emulate-user').each(function(idx, element) {
 		var data = {'href': $(element).attr('href')};
 		$(element).on('click', data, function(event) {
-			$.get(data.href).success(reloadBrowser);
+			$.get(data.href).then(reloadBrowser);
 			return false;
 		});
 	});
